'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.mapModule = mapModule;

var _path = require('path');

function getReplaceFunc(replaceFuncPath) {
    return require((0, _path.resolve)(process.cwd(), replaceFuncPath));;
}

function mapModule(source, file, state) {
    var opts = state.opts;
    var replaceContainer = getReplaceFunc(opts.replaceFunc);

    if (!replaceContainer) {
        throw new Error('Cannot find replace function file: ' + opts.replaceFunc);
    }

    var replaceHandlerName = opts.replaceHandlerName || 'default';
    var replace = replaceContainer[replaceHandlerName] || replaceContainer;

    // If the result is not a function, throw
    if (!replace || typeof replace !== 'function') {
        throw new Error('Cannot find replace handler in: ' + opts.replaceFunc + " with name: " + replaceHandlerName);
    }

    var result = replace(source, file, opts);
    if (result !== source) {
        return result;
    } else {
        return;
    }
}

exports.default = function (_ref) {
    var t = _ref.types;

    function transformRequireCall(nodePath, state) {
        if (!t.isIdentifier(nodePath.node.callee, { name: 'require' }) && !(t.isMemberExpression(nodePath.node.callee) && t.isIdentifier(nodePath.node.callee.object, { name: 'require' }))) {
            return;
        }

        var moduleArg = nodePath.node.arguments[0];
        if (moduleArg && moduleArg.type === 'StringLiteral') {
            var modulePath = mapModule(moduleArg.value, state.file.opts.filename, state);
            if (modulePath) {
                nodePath.replaceWith(t.callExpression(nodePath.node.callee, [t.stringLiteral(modulePath)]));
            }
        }
    }

    function transformImportCall(nodePath, state) {
        var moduleArg = nodePath.node.source;
        if (moduleArg && moduleArg.type === 'StringLiteral') {
            var modulePath = mapModule(moduleArg.value, state.file.opts.filename, state);
            if (modulePath) {
                nodePath.replaceWith(t.importDeclaration(nodePath.node.specifiers, t.stringLiteral(modulePath)));
            }
        }
    }

    return {
        visitor: {
            CallExpression: {
                exit: function exit(nodePath, state) {
                    return transformRequireCall(nodePath, state);
                }
            },
            ImportDeclaration: {
                exit: function exit(nodePath, state) {
                    return transformImportCall(nodePath, state);
                }
            }
        }
    };
};